@@ -1,11 +1,18 @@
-import subprocess,json,socket,time
-
+import subprocess, json, socket, time
 
 class WebBotMain:
     wait_timeout = 3  # seconds
     interval_timeout = 0.5  # seconds
     socket.setdefaulttimeout(5)
-    def __init__(self,port):
+
+    def __init__(self, port,browserName):
+        self.browserName= browserName
+        if browserName=='edge':
+            print('请注意')
+            print('接管Edge浏览器必须要有用户配置文件夹设置,谷歌和360安全浏览器则不必要')
+        elif browserName=='chrome':
+            print('请注意')
+            print('谷歌内核浏览器不能使用closepage，否则连接会断，不能往下操作, edge和360安全浏览器可以使用')
         self.port = port
         address_info = socket.getaddrinfo("127.0.0.1", port, socket.AF_INET, socket.SOCK_STREAM)[0]
         family, socket_type, proto, _, socket_address = address_info
@@ -21,8 +28,10 @@ class WebBotMain:
                 print(connect_error)
                 print('重试')
 
+
     @classmethod
-    def build(cls, address="127.0.0.1",port=26678,browserName='chrome',debugPort = 0,userDataDir="./UserData",browserPath='null',argument = "null") -> "WebBotMain":
+    def build(cls, address="127.0.0.1", port=26678, browserName='chrome', debugPort=0, userDataDir="./UserData",
+              browserPath='null', argument="null") -> "WebBotMain":
         """
         :param address: 字符串型，默认"127.0.0.1", webDriver服务地址。假如值为 "127.0.0.1"脚本会自动启动WebDriver.exe，如果是远程服务地址，用户需要手动启动WebDriver.exe 并且提供启动参数。
         :param port: 整型，默认52020，服务端口
@@ -37,7 +46,7 @@ class WebBotMain:
         subprocess.Popen(['WebDriver.exe', json.dumps(
             {"driverPort": port, "browserName": browserName, "debugPort": debugPort,
              "userDataDir": userDataDir, "browserPath": browserPath, "argument": argument})])
-        return WebBotMain(port)
+        return WebBotMain(port,browserName)
 
     def __send_data(self, *args) -> str:
         """
@@ -77,7 +86,7 @@ class WebBotMain:
     # ###############
     #   页面和导航   #
     # ###############
-    def goto(self,url) -> bool:
+    def goto(self, url) -> bool:
         """
         跳转url
         :param url: 字符串型, 网址
@@ -85,7 +94,7 @@ class WebBotMain:
         result = self.__send_data('goto', url)
         return result == "true"
 
-    def newPage(self,url) -> bool:
+    def newPage(self, url) -> bool:
         """
         新建tab页面并跳转到指定url
         :param url: 字符串型, 网址
@@ -113,7 +122,7 @@ class WebBotMain:
         """
         result = self.__send_data("refresh")
         return result == "true"
-        
+
     def getCurPageId(self) -> str:
         """
         获取当前页面ID
@@ -128,8 +137,8 @@ class WebBotMain:
         result = self.__send_data("getAllPageId").split('|')
 
         return result
-    
-    def switchPage(self,pageId) -> bool:
+
+    def switchPage(self, pageId) -> bool:
         """
         切换页面
         :param pageId: 字符串型, 要切换的页面ID
@@ -151,7 +160,7 @@ class WebBotMain:
         """
         result = self.__send_data("getCurrentUrl")
         return result
-        
+
     def getTitle(self):
         """
         获取页面标题
@@ -162,14 +171,14 @@ class WebBotMain:
     # #############
     #    IFrame   #
     # #############
-    def switchFrame(self,frameUrl)-> bool:
+    def switchFrame(self, frameUrl) -> bool:
         """
         通过url切换frame
         :param frameUrl: 字符串型，要切换frame的src链接。如果目标iframe没有scr连接，应当使用所在的iframe框架的链接
         """
         result = self.__send_data("switchFrame", frameUrl)
         return result == "true"
-        
+
     def switchMainFrame(self) -> bool:
         """
         切换主frame
@@ -180,32 +189,45 @@ class WebBotMain:
     # #############
     #    元素操作   #
     # #############
-    def clickElement(self,xpath) -> bool:
+    def clickElement(self, xpath, timeout=3.0) -> bool:
         """
         点击元素
         :param xpath: 字符串型，元素路径
         """
-        result = self.__send_data("clickElement", xpath)
-        return result == "true"
+        end_time = time.time() + timeout
+        while time.time() <= end_time:
+            result = self.__send_data("clickElement", xpath)
+            if result == "true":
+                return True
+        return False
 
-    def setElementValue(self,xpath, value) -> bool:
+    def setElementValue(self, xpath, value, timeout=3.0) -> bool:
         """
         设置编辑框内容
         :param xpath: 字符串型，元素路径
         :param value: 字符串型，输入的值
         """
-        result = self.__send_data("setElementValue", xpath, value)
-        return result == "true"
+        end_time = time.time() + timeout
+        while time.time() <= end_time:
+            result = self.__send_data("setElementValue", xpath, value)
+            if result == "true":
+                return True
+        return False
 
-    def getElementText(self,xpath) -> str:
+    def getElementText(self, xpath, timeout=3) -> str:
         """
         获取文本
         :param xpath: 字符串型，元素路径
         """
-        result = self.__send_data("getElementText", xpath)
-        return result
-    
-    def getElementOuterHTML(self,xpath):
+        end_time = time.time() + timeout
+        while time.time() <= end_time:
+            result = self.__send_data("getElementText", xpath)
+            if result != "null":
+                return result
+            elif result == "null":
+                return ''
+
+    def getElementOuterHTML(self, xpath):
         """
         获取outerHTML
         :param xpath: 字符串型，元素路径
@@ -213,14 +235,14 @@ class WebBotMain:
         result = self.__send_data("getElementOuterHTML", xpath)
         return result
 
-    def getElementInnerHTML(self,xpath) -> str:
+    def getElementInnerHTML(self, xpath) -> str:
         """
         获取innerHTML
         :param xpath: 字符串型，元素路径
         """
         result = self.__send_data("getElementInnerHTML", xpath)
         return result
-        
+
     def setElementAttribute(self, xpath, attributeName, value) -> bool:
         """
         设置属性值
@@ -230,49 +252,49 @@ class WebBotMain:
         """
         result = self.__send_data("setElementAttribute", xpath, attributeName, value)
         return result == "true"
-        
-    def getElementAttribute(self,xpath, attributeName) -> str:
+
+    def getElementAttribute(self, xpath, attributeName) -> str:
         """
         获取属性值
         :param xpath: 字符串型，元素路径
         :param attributeName: 指定的属性名
         """
-        result = self.__send_data("getElementAttribute", xpath,attributeName)
+        result = self.__send_data("getElementAttribute", xpath, attributeName)
         return result
-        
-    def getElementRect(self,xpath):
+
+    def getElementRect(self, xpath):
         """
         获取矩形位置
         :param xpath: 字符串型，元素路径
         """
         result = self.__send_data("getElementRect", xpath)
         return result
-        
+
     def isSelected(self, xpath) -> bool:
         """
         判断该元素是否选中
         :param xpath: 字符串型，元素路径
         """
-        result = self.__send_data("isSelected",  xpath)
+        result = self.__send_data("isSelected", xpath)
         return result == "true"
-    
-    def isDisplayed(self,xpath) -> bool:
+
+    def isDisplayed(self, xpath) -> bool:
         """
         判断该元素是否可见
         :param xpath: 字符串型，元素路径
         """
         result = self.__send_data("isDisplayed", xpath)
         return result == "true"
-    
-    def isEnabled(self,xpath) -> bool:
+
+    def isEnabled(self, xpath) -> bool:
         """
         判断元素是否可用
         :param xpath: 字符串型，元素路径
         """
         result = self.__send_data("isEnabled", xpath)
         return result == "true"
-        
-    def clearElement(self,xpath) -> bool:
+
+    def clearElement(self, xpath) -> bool:
         """
         清除元素值
         :param xpath: 字符串型，元素路径
@@ -283,23 +305,23 @@ class WebBotMain:
     # #############
     #   鼠标键盘   #
     # #############
-    def sendKeys(self,xpath, text) -> bool:
+    def sendKeys(self, xpath, text) -> bool:
         """
         发送文本
         :param xpath: 元素路径，如果元素不能设置焦点，应ClickMouse 点击锁定焦点输入
         :param text: 要输入的文本，例如sendKeys('//*[@id="kw"]', 'aibote\r'); aibote换行
         """
-        result = self.__send_data("sendKeys", xpath,text)
+        result = self.__send_data("sendKeys", xpath, text)
         return result == "true"
-        
-    def sendVk(self,vkCode)-> bool:
+
+    def sendVk(self, vkCode) -> bool:
         """
         发送Vk虚拟键
         :param vkCode: 整型，VK键值，仅支持 回退键:8  回车键:13  空格键:32  方向左键:37  方向上键:38  方向右键:39  方向下键:40  删除键:46
         """
         result = self.__send_data("sendVk", vkCode)
         return result == "true"
-    
+
     def clickMouse(self, x, y, msg) -> bool:
         """
         点击鼠标
@@ -307,9 +329,9 @@ class WebBotMain:
         :param y: 整型，纵坐标，非Windows坐标，页面左上角为起始坐标
         :param msg: 整型，单击左键:1  单击右键:2  按下左键:3  弹起左键:4  按下右键:5  弹起右键:6  双击左键:7
         """
-        result = self.__send_data("clickMouse", x, y,msg)
+        result = self.__send_data("clickMouse", x, y, msg)
         return result == "true"
-    
+
     def moveMouse(self, x, y):
         """
         移动鼠标
@@ -318,8 +340,7 @@ class WebBotMain:
         """
         result = self.__send_data("moveMouse", x, y)
         return result
-        
-        
+
     def wheelMouse(self, deltaX, deltaY, x=0, y=0) -> bool:
         """
         滚动鼠标
@@ -328,26 +349,30 @@ class WebBotMain:
         :param x: 整型，可选参数，鼠标横坐标位置， 默认为0
         :param y: 整型，可选参数，鼠标纵坐标位置， 默认为0
         """
-        result = self.__send_data("wheelMouse",deltaX, deltaY, x, y)
+        result = self.__send_data("wheelMouse", deltaX, deltaY, x, y)
         return result == "true"
-        
-    def clickMouseByXpath(self, xpath, msg) -> bool:
+
+    def clickMouseByXpath(self, xpath, msg, timeout=3) -> bool:
         """
         通过xpath点击鼠标(元素中心点)
         :param xpath: 字符串型，元素路径
         :param msg: 整型，单击左键:1  单击右键:2  按下左键:3  弹起左键:4  按下右键:5  弹起右键:6  双击左键:7
         """
-        result = self.__send_data("clickMouseByXpath", xpath, msg)
-        return result == "true"
-        
-    def moveMouseByXpath(self,xpath) -> bool:
+        end_time = timeout + time.time()
+        while time.time() <= end_time:
+            result = self.__send_data("clickMouseByXpath", xpath, msg)
+            if result == "true":
+                return True
+        return False
+
+    def moveMouseByXpath(self, xpath) -> bool:
         """
         通过xpath 移动鼠标(元素中心点)
         :param xpath: 字符串型，元素路径
         """
         result = self.__send_data("moveMouseByXpath", xpath)
         return result == "true"
-        
+
     def wheelMouseByXpath(self, xpath, deltaX, deltaY) -> bool:
         """
         通过xpath 滚动鼠标
@@ -361,7 +386,7 @@ class WebBotMain:
     # #############
     #     截图     #
     # #############
-    def takeScreenshot(self, xpath = None) ->str:
+    def takeScreenshot(self, xpath=None) -> str:
         """
         截图
         :param xpath: 可选参数，默认截取全屏, 如果指定元素路径，则截取元素图片。
@@ -376,7 +401,7 @@ class WebBotMain:
     # ##################
     #  alert/prompt弹窗  #
     # ##################
-    def clickAlert(self,acceptOrCancel, promptText = "") -> bool:
+    def clickAlert(self, acceptOrCancel, promptText="") -> bool:
         """
         点击警告框
         :param acceptOrCancel: 布尔型，true接受, false取消
@@ -385,8 +410,8 @@ class WebBotMain:
         """
         result = self.__send_data("clickAlert", acceptOrCancel, promptText)
         return result == "true"
-    
-    def getAlertText(self) ->str:
+
+    def getAlertText(self) -> str:
         """
         获取警告框内容
         """
@@ -396,7 +421,7 @@ class WebBotMain:
     # ###############
     #   cookie操作   #
     # ###############
-    def getCookies(self, url) ->str:
+    def getCookies(self, url) -> str:
         """
         获取指定url匹配的cookies
         :param url: 字符串型，指定的url http://或https:// 起头
@@ -404,7 +429,7 @@ class WebBotMain:
         """
         result = self.__send_data("getCookies", url)
         return result
-        
+
     def getAllCookies(self) -> str:
         """
         获取cookies
@@ -412,7 +437,7 @@ class WebBotMain:
         """
         result = self.__send_data("getAllCookies")
         return result
-        
+
     def setCookie(self, cookieParam) -> bool:
         """
         设置cookie
@@ -422,12 +447,12 @@ class WebBotMain:
             name、value和url必填参数，其他参数可选
         :return:
         """
-        #必填
+        # 必填
         name = cookieParam["name"]
         value = cookieParam["value"]
         url = cookieParam["url"]
         #
-        #可选
+        # 可选
         domain = path = sameSite = priority = sourceScheme = partitionKey = ""
         secure = httpOnly = sameParty = False
         expires = sourcePort = 0
@@ -454,14 +479,16 @@ class WebBotMain:
             sourcePort = cookieParam["sourcePort"]
         if "partitionKey" in cookieParam:
             partitionKey = cookieParam["partitionKey"]
-        strData = ("setCookie", name, value, url, domain, path, secure, httpOnly, sameSite, expires, priority, sameParty, sourceScheme, sourcePort, partitionKey)
+        strData = (
+        "setCookie", name, value, url, domain, path, secure, httpOnly, sameSite, expires, priority, sameParty,
+        sourceScheme, sourcePort, partitionKey)
         result = self.__send_data(strData)
         return result == "true"
 
     # ################
     #  注入JavaScript  #
     # ################
-    def executeScript(self,script) ->str:
+    def executeScript(self, script) -> str:
         """
         执行JavaScript 代码
         :param script: 字符串型，注入的js代码
